/////////////////////////////////////////////////////////////////////
// Performance Testing Application
//
// This program performs some simple performance testing for MQSeries
// Integrator
/////////////////////////////////////////////////////////////////////

//*******************************************************************
// Includes
//*******************************************************************
extern"C" 
{
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
	
// Includes for MQI  
#include <cmqc.h>
#include <MQSIrfheader.h>
}


// Includes for MQSeries C++
#include <imqi.hpp>
#include <fstream.h>
#include <iostream.h>

//*******************************************************************
// Constants
//*******************************************************************

// File details
#define FileName "driver.cfg"

// Options Building
#define AppGrpTag "OPT_APP_GRP"
#define MsgTypeTag "OPT_MSG_TYPE"
#define BLANK	" "

// Misc
#define PAD " "

//*******************************************************************
// Function Prototypes
//*******************************************************************

// Reads the perf.cfg parameter file and sets values
bool LoadParameters(ImqString &mgr,
					ImqString &pQueue,
					ImqString &gQueue,
					int &mSize,
					int &load,
					int &msgNo,
					int &cmitEvery,
					int &timeEvery,
					ImqString &appGrp,
					ImqString &msgType,
					ImqString &msgData,
					ImqString &logFile);

/////////////////////////////////////////////////////////////////////
// Start main program
/////////////////////////////////////////////////////////////////////
void main()
{
	// Variables
	ImqQueueManager mgr;				// Queue Manager with both queues on
	ImqQueue pQueue;					// Put queue
	ImqQueue gQueue;					// Get queue
	ImqGetMessageOptions gmo;			// Get message options
	ImqPutMessageOptions pmo;			// Put message options
	ImqString mgrName;					// Queue manager name	
	ImqString pQName;					// Put queue name	
	ImqString gQName;					// Get queue name 
	ImqString appGrp;					// Application group
	ImqString msgType;					// Message type
	ImqString msgData;					// Message data
	ImqString options;					// NEON options
	ImqString logFile;					// Name of Logfile
	int mSize;							// Message size (bytes)
	int load;							// Initial put queue message load
	int msgNo;							// Number of messages for test
	int cmitEvery;						// Number of messsages put/get before commit
	int timeEvery;						// Number of messages before displaying time
	MQRFH NeonHDR = {MQRFH_DEFAULT};	// NEON header
	char* MsgPtr;						// Pointer to whole data block
	ImqMessage msg;						// MQ Put Message
	ImqMessage gMsg;					// MQ Gut Message
	time_t start;						// Time before 1st get/put
	time_t lap;							// Time at interval requested
	int getCount=0;						// Number of msgs processed
	ofstream Log;						// Logfile

/////////////////////////////////////////////////////////////////////
// Load Parameters From File
/////////////////////////////////////////////////////////////////////

	if(!LoadParameters(mgrName,pQName,gQName,mSize,load,msgNo,
				   cmitEvery,timeEvery,appGrp,msgType,
				   msgData,logFile)) {
		cerr << "Error with cfg file.  Check presence and correctness" 
			 << endl;
		exit(0);
	}

/////////////////////////////////////////////////////////////////////
// Connect to QMgr
/////////////////////////////////////////////////////////////////////
	
	mgr.setName(mgrName);
	
	if ( ! mgr.connect( ) ) {
		
		// stop if it failed 
		cerr << "Connect to queue manager ended with reason code "
             << (int)mgr.reasonCode( ) << "- program ending" << endl;
		exit(0);
	}

/////////////////////////////////////////////////////////////////////
// Open Get Queue
/////////////////////////////////////////////////////////////////////

	// Set name
	gQueue.setName(gQName);

	// Associate with qmgr
	gQueue.setConnectionReference( mgr );

	// Open
	gQueue.setOpenOptions( MQOO_INPUT_AS_Q_DEF ); // open gQueue for input        
	gQueue.open();
	
	// Report reason, if any; stop if failed
	if ( gQueue.reasonCode( ) ) {
		cerr << "WARNING - open get queue ended with reason code "
             << (int)gQueue.reasonCode() << endl;
	}
	
	if ( gQueue.completionCode( ) == MQCC_FAILED ) {
		cerr << "Unable to open get queue - program ending" << endl;
		exit(0);
	}

/////////////////////////////////////////////////////////////////////
// Open Out Queue
/////////////////////////////////////////////////////////////////////

	// Set name
	pQueue.setName(pQName);

	// Associate with qmgr
	pQueue.setConnectionReference( mgr );

	// Open
	pQueue.setOpenOptions( MQOO_OUTPUT ); // open pQueue for output        
	pQueue.open();
	
	// Report reason, if any; stop if failed
	if ( pQueue.reasonCode( ) ) {
		cerr << "WARNING - open put queue ended with reason code "
             << (int)pQueue.reasonCode() << endl;
	}
	
	if ( pQueue.completionCode( ) == MQCC_FAILED ) {
		cerr << "Unable to open put queue - program ending" << endl;
		exit(0);
	}


/////////////////////////////////////////////////////////////////////
// Open Logfile
/////////////////////////////////////////////////////////////////////
	Log.open(logFile);
	if(!Log.good()) {
		cerr << "Logfile " << FileName << " could not be opened" << endl;
		exit(0);
	}

/////////////////////////////////////////////////////////////////////
// Build Options String
/////////////////////////////////////////////////////////////////////

	options+=AppGrpTag;
	options+=BLANK;
	options+=appGrp;
	options+=BLANK;
	options+=MsgTypeTag;
	options+=BLANK;
	options+=msgType;

/////////////////////////////////////////////////////////////////////
// Pad Data To Correct Length
/////////////////////////////////////////////////////////////////////

	while (msgData.length()!=mSize) {
		msgData+=PAD;
	}

/////////////////////////////////////////////////////////////////////
// Build Message
/////////////////////////////////////////////////////////////////////

	// Calculate data lengths
	int HdrLength = sizeof(MQRFH);
	int OpLength  = options.length();
	int DataLength= msgData.length();
	int totSize = HdrLength+OpLength+DataLength;
	
	// Setup Neon Header
	strncpy(NeonHDR.Format,MQFMT_STRING,sizeof(MQFMT_STRING));
	NeonHDR.StrucLength=HdrLength+OpLength;
	
	// Build message
	MsgPtr = (char*) malloc(totSize);
	
	// Copy in data
	memcpy(MsgPtr, &NeonHDR, HdrLength);
	memcpy((MsgPtr+HdrLength), (char*)options, OpLength);
	memcpy((MsgPtr+HdrLength+OpLength), (char*)msgData, DataLength);
	msg.useFullBuffer( MsgPtr, totSize );
	msg.setFormat( MQFMT_RF_HEADER );	// Neon Header

/////////////////////////////////////////////////////////////////////
// Set Put/Get Message Options
/////////////////////////////////////////////////////////////////////
	
	pmo.setOptions(MQPMO_SYNCPOINT);
	gmo.setOptions(MQGMO_WAIT | MQGMO_SYNCPOINT);
	gmo.setWaitInterval(30000);

/////////////////////////////////////////////////////////////////////
// Put Initial Message Load
/////////////////////////////////////////////////////////////////////

	for(int i=0;i<load;i++) {
		// Reset IDs
		msg.setMessageId( );
		msg.setCorrelationId( );
		
		if ( ! pQueue.put( msg ) ) {
			
			// Report reason, if any 
			cerr << "WARNING - initial load message put ended with reason code "
				<< (int)pQueue.reasonCode( ) << endl;
		}
	}

	// Commit initial put load
	if(!mgr.commit()) {
		cerr << "Commit of initial message load failed"
			 << endl;
		exit(0);
	}

/////////////////////////////////////////////////////////////////////
// Loop through requested amount of get/puts
/////////////////////////////////////////////////////////////////////

	// Initialise timing
	time(&start);

	// Start looping
	for(i=0;i<msgNo;i++) {
		
		// Check wether to commit
		if ((i % cmitEvery) == 0) {
			if(!mgr.commit()) {
				cerr << "Commit during loop failed"
					 << endl;
				exit(0);
			}
		}

		// Reset IDs
		gMsg.setMessageId( );
		gMsg.setCorrelationId( );

		// Syncpoint get		
		if (!gQueue.get(gMsg,gmo)) {
			if ((int)gQueue.reasonCode() == MQRC_NO_MSG_AVAILABLE) {
				// Test is over
				getCount++;
				time(&lap);
				if (getCount % timeEvery == 0) {
					Log << timeEvery << " msgs in " << difftime(lap,start)
						<< " seconds" << endl;
				}
				cout << "Test finished" << endl;
			}
			else {
				cerr << "Get during loop failed"
					 << gQueue.reasonCode() << endl;
				exit(0);
			}
		}
		else
		{
			getCount++;
			time(&lap);
			if (getCount % timeEvery == 0) {
				Log << timeEvery << " msgs in " << difftime(lap,start)
					 << " seconds" << endl;
			// Re-initialise timing
			time(&start);
			}
		}
		
		// Syncpoint Put
		
		// Reset IDs
		msg.setMessageId( );
		msg.setCorrelationId( );
		
		// Put message
		if ( ! pQueue.put( msg ) ) {
			
			// Report reason, if any 
			cerr << "WARNING - put during loop ended with reason code "
				<< (int)pQueue.reasonCode( ) << endl;
		}
	}

/////////////////////////////////////////////////////////////////////
// Close Logfile
/////////////////////////////////////////////////////////////////////
	Log.close();

/////////////////////////////////////////////////////////////////////
// Close put queue
/////////////////////////////////////////////////////////////////////
	if ( ! pQueue.close( ) ) {
		
		// Report reason, if any     
		cerr << "WARNING - close put queue ended with reason code "
             << (int)pQueue.reasonCode() << endl;
	}

/////////////////////////////////////////////////////////////////////
// Close get queue
/////////////////////////////////////////////////////////////////////
	if ( ! gQueue.close( ) ) {
		
		// Report reason, if any     
		cerr << "WARNING - close get queue ended with reason code "
             << (int)gQueue.reasonCode() << endl;
	}
	
/////////////////////////////////////////////////////////////////////
// Disconnect from queue manager
/////////////////////////////////////////////////////////////////////
	if ( ! mgr.disconnect( ) ) {
		
		// Report reason, if any     
		cerr << "WARNING - disconnect from queue manager ended with reason code "
             << (int)mgr.reasonCode() << endl;
	}

/////////////////////////////////////////////////////////////////////
// END
/////////////////////////////////////////////////////////////////////
}


//*******************************************************************
// Function Definitions
//*******************************************************************

// Reads the perf.cfg parameter file and sets values
bool LoadParameters(ImqString &mgr,
					ImqString &pQueue,
					ImqString &gQueue,
					int &mSize,
					int &load,
					int &msgNo,
					int &cmitEvery,
					int &timeEvery,
					ImqString &appGrp,
					ImqString &msgType,
					ImqString &msgData,
					ImqString &logFile)
{
	// Variables
	ifstream file;
	char data[5000];

	// Attempt to open file
	file.open(FileName, ios::in | ios::nocreate);
	if(!file.good()) {
		cerr << "File " << FileName << " could not be opened" << endl;
		return(false);
	}
	else {
		// Read in values
		file.getline(data,5000,'\n');
		mgr.pasteIn(data,strlen(data));
		file.getline(data,5000,'\n');
		pQueue.pasteIn(data,strlen(data));
		file.getline(data,5000,'\n');
		gQueue.pasteIn(data,strlen(data));
		file.getline(data,5000,'\n');
		appGrp.pasteIn(data,strlen(data));
		file.getline(data,5000,'\n');
		msgType.pasteIn(data,strlen(data));
		file.getline(data,5000,'\n');
		msgData.pasteIn(data,strlen(data));
		file.getline(data,5000,'\n');
		logFile.pasteIn(data,strlen(data));
		file >> mSize 
			 >> load 
			 >> msgNo
			 >> cmitEvery
			 >> timeEvery;
	}
	file.close();
	return true;
}

